﻿TUI.Core.create({
    name:"TUI.map.BaiduImpl",
    extend:TUI.map.BaseMap,
    init:function () {
        this.createControl();                                // 创建地图控件
    },
    methods:{
        // 创建地图对象
        createMap:function () {
            var maptypeid = BMAP_NORMAL_MAP;
            if (maptype == 1)
                maptypeid = BMAP_HYBRID_MAP;
            var mapOptions = {
                mapType:maptypeid
            };
            var map = this._map = new BMap.Map(this._mapPanel, mapOptions);              // 创建Map实例
            map.addControl(new BMap.NavigationControl());                 // 添加平移缩放控件
            map.addControl(new BMap.ScaleControl());                      // 添加比例尺控件
            map.addControl(new BMap.OverviewMapControl({isOpen:false}));// 添加缩略地图控件
            map.enableScrollWheelZoom();								      // 启用滚轮放大缩小
            this._rectangleZoom = new BMapLib.RectangleZoom(map, {autoClose:true});	// 拉框放大工具
            this._distanceTool = new BMapLib.DistanceTool(map); 						// 测距工具
            this.setCenter( 110.397, 36.917, 5 );
        },
        // 拉框放大
        dragMax: function() {
            this._rectangleZoom._opts.zoomType = 0;
            this._rectangleZoom.open();
        },
        // 拉框缩小
        dragMin: function() {
            this._rectangleZoom._opts.zoomType = 1;
            this._rectangleZoom.open();
        },
        // 移动地图
        moveMap: function() {
        },
        // 测距
        range: function() {
            this._distanceTool.open();
        },
        // 基本函数，显示全国图
        fullMap:function(){
            this.setCenter( 110.397, 36.917, 5 );
        },
        // 设置中心点
        setCenter: function( lng, lat, zoom ) {
            var lngLat = this._newLngLat( lng, lat );
            this._map.centerAndZoom( lngLat, zoom );
        },
        // 锁定视窗
        lockWindowByLngLat: function( lngLat ) {
            var lo = lngLat.lng, la = lngLat.lat;
            var bounds = this._map.getBounds();
            var left  = bounds.getSouthWest().lng ;	//西南 经度
            var down  = bounds.getSouthWest().lat ;	//西南 纬度
            var right = bounds.getNorthEast().lng ;	//东北 经度
            var up    = bounds.getNorthEast().lat ;	//东北 纬度);
            if ( lo < left || lo > right || la < down || la > up ) {
                this._map.setCenter( this._newLngLat( lo, la ) );
            }
        },
        // 最佳视窗效果
        _setFitview: function( overlays ) {
            var self = this, lngLats = [];
            for( var i = 0; i < overlays.length; i ++ ){
                var item = overlays[i];
                if ( item instanceof BMap.Marker ) {
                    lngLats.push( item.getPosition() );
                }
                else if ( item instanceof BMap.Polyline || item instanceof BMap.Polygon ) {
                    lngLats = lngLats.concat( item.getPath() );
                }
                else if ( item instanceof BMap.Point ) {
                    lngLats.push( item );
                }
            }
            var viewport = this._map.getViewport( lngLats );
            this._map.setViewport( viewport );
        },
        /////////////////////////////////////内部私有函数///////////////////////////////////
        // 添加具体的覆盖物实例
        _addOverlay: function( overlay ) {
            this._map.addOverlay(overlay);
        },
        // 移除具体的覆盖物实例
        _removeOverlay: function( overlay ) {
            this._map.removeOverlay(overlay);
        },
        // 根据经度，纬度创建位置点
        _newLngLat: function( lng, lat ) {
            return new BMap.Point( lng, lat );
        },
        // 创建Marker
        _newMarker: function( opt ) {
            var marker = new BMap.Marker( opt.lngLat );
            if( opt.icon && opt.size ){   // 如果指明了图标与尺寸
                var _icon = new BMap.Icon( opt.icon , new BMap.Size( opt.size.x,opt.size.y ) );
                marker.setIcon( _icon );
            }
            if(opt.label && opt.offset ){  // 如果指明了文本及偏移地址
                var _label = new BMap.Label( opt.label, {offset: new BMap.Size( opt.offset.x, opt.offset.y ) } );
                marker.setLabel( _label);
            }
            return marker;
        },
        // 刷新Makert上的信息
        _refreshMarker:function( marker, opt ){
            if( marker ){
                if( opt.lngLat ){   // 如果指明了位置，就刷新位置
                    marker.setPosition(opt.lngLat);
                }
                if( opt.icon && opt.size ){   // 如果指明了图标与尺寸
                    var _icon = new BMap.Icon( opt.icon , new BMap.Size( opt.size.x,opt.size.y ) );
                    marker.setIcon( _icon );
                }
                if(opt.label){  // 如果指明了文本及偏移地址
                    marker.getLabel().setContent(opt.label);
                }
            }
        },
        // 根据经度，纬度创建线
        _newPolyline: function( opt ){
            return new BMap.Polyline( opt.lngLat,{strokeColor: opt.color,strokeOpacity: 0.7,strokeWeight: 5});
        },

        // 绑定事件
        _bind:function(elem, event, fun){
            elem.addEventListener(event, fun);    // 注册事件
        },

        // 释放事件
        _unbind:function(elem, event, fun){
            elem.removeEventListener(event, fun);  // 释放事件
        },

        // 获取对象的轨迹
        _getPath: function(poly){
            return poly.getPath();
        },

        // 注册车辆信息窗口
        _addInfoWindow: function( marker, car, pos) {
            var self = this;
            this._safeBind(marker,"click", function(){
                if( self._infoWin ){   // 只开一个窗口
                    self._infoWin.close();
                }
                var html = self.getDetailCarTip(car.carNO, pos);
                var iw = new BMap.InfoWindow( html, {height: 150} );
                this.openInfoWindow( iw );
                self.getAddress(pos.lo, pos.la, TUI.Event._id());
                self._infoWin = iw;                          // 保存下窗体
                self._infoWin.carId = car.carId;              // 并且保存下窗体对应的空间对象
            });
        },

        // 刷新车辆信息窗口
        _refreshInfoWindow:function(marker, car, pos){
            var self = this;
            var iw = self._infoWin;
            if( iw ){
                var html = self.getDetailCarTip(car.carNO, pos);
                iw.setContent(html);
                self.getAddress(pos.lo, pos.la, TUI.Event._id());
            }
        },

        /////////////////////////////////////空间对象相关///////////////////////////////////
        // 绘制空间对象
        drawGeo:function(type, points, editable ){
            var self = this;
            var path = [];         // 已经接收的点
            var drawPath = null;  // 显示需要的点

            this._cleanBind(self._map);     // 清空所有事件
            this.cleanGeo();                // 清空对象

            // 如果有坐标，说明是更新，直接生成可以编辑的对象
            if( points && points.length > 0 ){
                var lngLats = this._parsePoints(points);
                _createGeo(lngLats, editable);   // 创建空间对象
                if( type == 0 )
                    this.setCenter( lngLats[0].lng, lngLats[0].lat, 12 );
                else
                    this._setFitview([this._geo]);  // 居中显示
            }
            // 否则，注册事件重新绘制
            else{
                self._map.setDefaultCursor('crosshair');
                self._safeBind( self._map, 'click', _clickCB);
                self._safeBind( self._map, 'mousemove', _moveCB);
                self._safeBind( self._map, 'rightclick', _rclickCB);
            }
            // 左键单击事件
            function _clickCB(event){
                path.push(event.point);                        // 往路径中加点
                drawPath = path.concat(path[path.length -1]);  // 显示的点总要多一个
                // 左键时，如果对象为空
                if( self._geo == null ){
                    _createGeo(drawPath, false);    // 以第一次单击位置创建对象
                }
                else{
                    if( type == 0 ){
                        self._geo.setPosition(event.point);
                    }
                    else{
                        self._geo.setPath(drawPath);
                    }
                }
            }
            // 鼠标移动事件
            function _moveCB(event) {
                if( self._geo && type != 0 && drawPath.length > 1 ){
                    self._geo.setPositionAt(drawPath.length -1, event.point); // 修改最后一个显示点的位置
                }
            }
            // 右击事件
            function _rclickCB(event) {
                self._cleanBind(self._map);
                self._map.setDefaultCursor("url(../images/openhand.cur)");
                if( self._geo  && type != 0 && path.length > 1){
                    self._geo.setPath(path);
                }
            }
            // 创建空间对象
            function _createGeo(lngLats, canEdit){
                // 如果是点
                if( type == 0 ){
                    self._geo = self._newMarker({
                        lngLat: lngLats[0],                     // 坐标
                        icon:"../images/marker.png",         // 图标
                        size:{x:25,y:25},                       // 图标大小
                        label: lg.markTip,            // 标题
                        offset:{x:27,y:6}                      // 标题偏移
                    });

                    self._addOverlay(self._geo);  // 添加到地图
                    if( canEdit )
                        self._geo.enableDragging();   // 允许拖动
                }
                // 如果是线
                else if( type == 1 ){
                    self._geo = new BMap.Polyline( lngLats, {
                        strokeColor:"#FF0000",
                        fillColor:"#FF0000",
                        strokeWeight:2,
                        strokeOpacity:0.7,
                        fillOpacity:0.2,
                        strokeStyle:"dashed"
                    });

                    self._addOverlay(self._geo);  // 添加到地图
                    if( canEdit )
                        self._geo.enableEditing();
                }
                // 如果是面
                else if( type == 2 ){
                    self._geo = new BMap.Polygon( lngLats, {
                        strokeColor:"#FF0000",
                        fillColor:"#FF0000",
                        strokeWeight:2,
                        strokeOpacity:0.7,
                        fillOpacity:0.2,
                        strokeStyle:"dashed"
                    });

                    self._addOverlay(self._geo);  // 添加到地图
                    if( canEdit )
                        self._geo.enableEditing();
                }
            }
        }
    }
});